//-------------------------  1991-1994 by James G. Stout --------------------------
// File		: cdefGBox.c
// Date		: September 1,1991
// Author	: Jim Stout
// 			:
// Purpose	: "Group Box" CDEF
// 			: A very simple CDEF that draws a titled box. It supports
// 			: 3 variations - grayLine, ctl3D, insetBox, and useWindFont (1,2, 4 & 8)
// 			:
// 			: Pretty straight forward code here, only trick here is to use the 
// 			: contrlMax value as the height of the control.  This is done to keep
// 			: the Control Manager from thinking the control overlays the controls
// 			: you want inside the GroupBox.  Set the real control rect to some value
// 			: sufficient to see the title in your resource editor and set contrlMax
// 			: to the height of the box you want drawn.
//			:
// ** IMPORTANT INFORMATION **
//			:
//			: Since the Control Manager doesn't know about the extra drawing done
//			: by this control, some extra handling is required for update events.
//			: Just calling UpdtControl() will not suffice since there could be a
//			: portion of this control that needs to be drawn that is outside the
//			: "official" control rect.
//			:
//			: So, make sure you call Draw1Control() in response to an update for
//			: any control that uses this CDEF.  Calling DrawControls is ok too, but
//			: will draw all of the controls in the window.
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------
//#define _DEBUGCDEF
#include <SetUpA4.h>

#include "colorCDEF.h"
#include "grayCDEF.h"
#include "miscCDEF.h"

#include "cdefGBox.h"

#ifdef _DEBUGCDEF
pascal long CDmain	(short, ControlHandle, short, long);

pascal long CDmain	(short varCode, ControlHandle hCtl, short message, long param)
#else
//==================================================================================
// CDEF entry point
//==================================================================================

pascal long main	(short varCode, ControlHandle hCtl, short message, long param)
#endif
{
	gbDataHandle	hGB;
	Rect			r;
	
	RememberA0();
	SetUpA4();
	
	HLock((Handle)hCtl);
	
	switch(message) {
	
		case initCntl:
			hGB = (gbDataHandle)NewHandle(sizeof(gbData));
			if(hGB) {
				(**hGB).realHt = (**hCtl).contrlMax;		// true height of control
				
				r = (**hGB).titleRect = (**hCtl).contrlRect;// we'll adjust this later
				r.bottom = r.top + (**hGB).realHt+1;		// for font size
				r.right++;									// allow for 3D shadow
				
				InvalRect(&r);								// fool ControlMgr since
															// since we draw outside
															// the true controlRect.
																	
				(**hCtl).contrlData = (Handle)hGB;			// save our data
			}
		break;
		
		case dispCntl:
			hGB = (gbDataHandle)(**hCtl).contrlData;
			if(hGB) {
				r = (**hCtl).contrlRect;
				r.bottom = r.top + (**hGB).realHt+1;		// can't rely on Control Mgr
				r.right++;									// to erase our "fake" rect
				EraseRect(&r);								// or the 3D shadow.
				DisposeHandle((Handle)hGB);
				(**hCtl).contrlData = 0;
			}
		break;
		
		case drawCntl:
			doDraw(hCtl, varCode);
		break;
		
		case calcCRgns:
			RectRgn((RgnHandle)(param & 0x7fffffffL), &(*hCtl)->contrlRect);
		break;
		
		case calcCntlRgn:
		case calcThumbRgn:
			RectRgn((RgnHandle)(param), &(*hCtl)->contrlRect);
		break;
	}
	
	HUnlock((Handle)hCtl);
	
	RestoreA4();
	return (0L);											// this control does nothing
}

//==================================================================================
// as it says, the draw routine for the group box
//==================================================================================
void doDraw(ControlHandle hCtl, short varCode)
{
	short				titleWidth, titleHt;
	Rect				r, titleRect, boxRect;
	GrafPtr				savePort;
	PenState			savePen;
	RgnHandle			saveClip;
	FontInfo			f;
	short				txF,txS,pixDepth;
	RGBColor			saveFore,saveBack,grayColor;
	RGBColor			rgbWhite = {-1, -1, -1};
	Boolean				inColor = false, bgInColor = false, haveGray = false;
	gbDataHandle		hGB;
	
//----------------------------------------------------------------------------------
// get our private data 
//----------------------------------------------------------------------------------

	hGB = (gbDataHandle)(**hCtl).contrlData;
	if(!hGB)
		return;											// oops!

//----------------------------------------------------------------------------------
// setup our drawing environment
//----------------------------------------------------------------------------------

	GetPort(&savePort);
	GetPenState(&savePen);	
	saveClip = NewRgn();								// save clipping region	
	GetClip(saveClip);
		
	if(!(varCode & useWindFont)) {						// use system font			
		txF = savePort->txFont;
		txS = savePort->txSize;
		savePort->txFont = SysFontFam;
		savePort->txSize = SysFontSize;
	}
	
	if(varCode & grayLine) {
		PenPat( (ConstPatternParam) "\x55\xAA\x55\xAA\x55\xAA\x55\xAA");
	}
	
//----------------------------------------------------------------------------------
// Clear the title box at the top so that any font changes don't leave junk
// lying around.
//----------------------------------------------------------------------------------

	EraseRect(&(**hGB).titleRect);						// clear previous rect	
	
	GetFontInfo(&f);									// get font metrics		
	titleRect = (**hGB).titleRect;						// calc new title rect
	titleRect.bottom = titleRect.top + 
						f.ascent + f.descent;
	(**hGB).titleRect = titleRect;						// save new title rect
	
	f.ascent+=1;										// fudge a bit so the
	titleRect.bottom = titleRect.top + 2*(f.ascent/2);	// title lines up
	titleHt = titleRect.bottom-titleRect.top;			// nicely

//----------------------------------------------------------------------------------
// calculate the real rect for the control
//----------------------------------------------------------------------------------
	
	boxRect = (**hGB).titleRect;
	boxRect.bottom = boxRect.top + (**hGB).realHt;		// full control rect	
	r = boxRect;	
	boxRect.top+=titleHt/2;								// box halves title area
								
	InsetRect(&r, -1, -1);								// allow for 3D shadow
	ClipRect(&r);										// set new clipping region

//----------------------------------------------------------------------------------
// do color stuff
//----------------------------------------------------------------------------------

	pixDepth = getPixDepth(&r);							// are we in color?
	if(pixDepth > 2) {									// yes
		inColor = true;
		saveColors(&saveFore, &saveBack);
		setPartColor(hCtl, cFrameColor, true);
		bgInColor = true;
		if(saveBack.red == 65535 &&						// is bg white?
			saveBack.green == 65535 &&
			saveBack.blue == 65535)
			bgInColor = false;							// no 3D effects
	}
	
	if(inColor) {
		if(bgInColor && 
			varCode & ctl3D && 							// do 3D stuff
			!(varCode & insetBox)) {					// but not for inset box
			RGBForeColor(&rgbWhite);
			OffsetRect(&boxRect, 1, 1);
			FrameRect(&boxRect);						// white shadow
			OffsetRect(&boxRect, -1, -1);
			setPartColor(hCtl, cFrameColor, true);		// groupBox color
		}
			
		if((**hCtl).contrlHilite == 0xFF) {				// inactive control
			if(haveGray = getGray(&grayColor))			// grayed cFrameColor
				RGBForeColor(&grayColor);
		}
	}
	
//----------------------------------------------------------------------------------
// finally, we can start drawing the control
//----------------------------------------------------------------------------------

	if(bgInColor && varCode & insetBox) {
		ForeColor(whiteColor);
		MoveTo(boxRect.right, boxRect.top);
		LineTo(boxRect.right, boxRect.bottom);
		LineTo(boxRect.left, boxRect.bottom);
		setPartColor(hCtl, cFrameColor, true);
		getGray(&grayColor);
		RGBForeColor(&grayColor);
		LineTo(boxRect.left, boxRect.top);
		LineTo(boxRect.right, boxRect.top);
	}
	else
		FrameRect(&boxRect);							// frame the groupbox
	
	titleWidth = StringWidth((*hCtl)->contrlTitle) + 10;
	
	if(titleWidth > 10) {								// i.e. we have a title	
		titleRect.left += 10;
		titleRect.right = titleRect.left + titleWidth;
		EraseRect(&titleRect);							// clear title area		
		if(inColor) {
			if(bgInColor && varCode & ctl3D) {			// do 3D stuff
				RGBForeColor(&rgbWhite);
				MoveTo(titleRect.left + 6,boxRect.top+titleHt/2-f.leading+1);
				DrawString((*hCtl)->contrlTitle);		// embossed title in white	
			}
			setPartColor(hCtl, cTextColor, true);		// get fgColor
		
			if(haveGray) {								// must be inactive
				getGray(&grayColor);					// grayed cTextColor
				RGBForeColor(&grayColor);
			}
		}
	
		MoveTo(titleRect.left + 5,boxRect.top+titleHt/2-f.leading);
		DrawString((*hCtl)->contrlTitle);				// draw the title in fgColor			
	}
	
//----------------------------------------------------------------------------------
// gray out the old, System 6 way if we didn't get a gray color
//----------------------------------------------------------------------------------

	if(!haveGray && (**hCtl).contrlHilite == 0xFF) {	// no gray but inactive
		
		PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
		PenMode(patBic);
		FrameRect(&boxRect);							// gray the groupbox
		PaintRect(&(**hGB).titleRect);					// gray the title
	}
		
//----------------------------------------------------------------------------------
// restore drawing environment
//----------------------------------------------------------------------------------

	if(inColor) {
		restoreColors(&saveFore, &saveBack);
	}
	
	if(!(varCode & useWindFont)) {						// restore font & size	
		savePort->txFont = txF;
		savePort->txSize = txS;
	}
	SetClip(saveClip);
	DisposeRgn(saveClip);
	SetPenState(&savePen);
}