//-------------------------  1994 by James G. Stout ------------------------------
//	File	: cdefVSlider.c
//	Date	: April 6, 1994
//	Author	: Jim Stout
//			:
//	Purpose	: A slider CDEF modeled after the one in the Sound Control panel
//			:
//			: see cdefVSlider.h for more detail and variation codes
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------
//#define _DEBUGCDEF					// comment this to build CDEF

#include <QDOffscreen.h>
#include <GestaltEqu.h>
#include <SetUpA4.h>

#include "grayCDEF.h"
#include "colorCDEF.h"
#include "miscCDEF.h"
#include "qdCDEF.h"

#include "cdefVslider.h"

#ifdef _DEBUGCDEF
pascal long CDmain (short, ControlHandle, short, long);

pascal long CDmain (short varCode, ControlHandle theControl, short message, long param)
#else

//==================================================================================
// CDEF entry point
//==================================================================================
pascal long main (short varCode, ControlHandle theControl, short message, long param)
#endif
{
	long	ret = 0L;
	
	RememberA0();
	SetUpA4();
	
	HLock((Handle)theControl);
	if((**theControl).contrlData)
		HLock((**theControl).contrlData);
	
	switch(message) {
		case initCntl:
			doInit(theControl, varCode);
		break;
		case dispCntl:
			doDisp(theControl);
		break;
		case drawCntl:
			if ((**theControl).contrlVis != 0 &&
				((WindowPeek)(**theControl).contrlOwner)->visible) {
				doDraw(theControl, varCode);
			}
		break;
		case testCntl:
			ret = doTest (theControl, varCode, param);
		break;
		case calcCRgns:
			RectRgn((RgnHandle)(param & 0x7fffffffL), &(**theControl).contrlRect);
		break;
		case calcCntlRgn:
		case calcThumbRgn:
			RectRgn((RgnHandle)(param), &(**theControl).contrlRect);
		break;
	}

	if((**theControl).contrlData)
		HUnlock((**theControl).contrlData);
	HUnlock((Handle)theControl);
	
	RestoreA4();
	
	return (ret);
}

//==================================================================================
//	Initialize our control data
//==================================================================================

static long doInit (ControlHandle theControl, short varCode)
{
	short		scaleSize;
	Rect		scaleRect;
	CDEFHandle	hCDEF;
	
//----------------------------------------------------------------------------------
// check for a reasonably sized scale
//----------------------------------------------------------------------------------
	
	scaleSize = (**theControl).contrlRfCon;
	if(scaleSize < 2 || scaleSize > 20)			// must be at least 2 units high
		scaleSize = 7;							//  7 is like the Sound Control panel
		
//----------------------------------------------------------------------------------
// force the scale rect to the size we want, it'll be clipped to the control Rect
//----------------------------------------------------------------------------------
	
	scaleRect = (**theControl).contrlRect;
	scaleRect.right = scaleRect.left + 42;
	scaleRect.bottom = scaleRect.top + scaleSize*12+21;
	
//----------------------------------------------------------------------------------
// force the min & max as well
//----------------------------------------------------------------------------------
	
	(**theControl).contrlMin = 0;
	(**theControl).contrlMax = scaleSize*12;	// number of pixels in scale
	(**theControl).contrlRfCon = 0;
	
	hCDEF = (CDEFHandle)NewHandle(sizeof(CDEFData));
	if(hCDEF) {
		(**hCDEF).pDepth = 1;					// filled in in doDraw()
		(**hCDEF).offPort = 0;
		(**hCDEF).scalePort = 0;
		(**hCDEF).scaleSize = scaleSize;
		(**hCDEF).scaleRect = scaleRect;
		(**hCDEF).firstClick = 2;
		(**hCDEF).qdVers = getQDVers();
		
		(**theControl).contrlData = (Handle)hCDEF;
		HLock((**theControl).contrlData);
	}
	else
		(**theControl).contrlData = 0;
}

//==================================================================================
//	Get rid of our control data
//==================================================================================

static long doDisp		(ControlHandle theControl)
{
	CDEFHandle	hCDEF;
	BitMap		*theBits;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		if((**hCDEF).qdVers < gestalt32BitQD12) {		// offport is bitmap
			theBits = &((GrafPtr)(**hCDEF).offPort)->portBits;
			DisposePtr((*theBits).baseAddr);
			ClosePort((GrafPtr)(**hCDEF).offPort);
			DisposePtr((Ptr)(**hCDEF).offPort);
			
			theBits = &((GrafPtr)(**hCDEF).scalePort)->portBits;
			DisposePtr((*theBits).baseAddr);
			ClosePort((GrafPtr)(**hCDEF).scalePort);
			DisposePtr((Ptr)(**hCDEF).scalePort);
		}
		else {
			DisposeGWorld((**hCDEF).offPort);			// offPort is GWorld
			DisposeGWorld((**hCDEF).scalePort);
		}
			
		HUnlock((Handle)hCDEF);
		DisposeHandle((Handle)hCDEF);
		(**theControl).contrlData = 0;
	}
}

//==================================================================================
//	Test to see if the mouse was pressed in the control thumb and 
//  drag accordingly
//==================================================================================

static long doTest (ControlHandle theControl, short varCode, long param)
{
	Rect	rThumb, rScale;
	Point	p;
	long	ret=0,secs;
	short	partCode;
	CDEFHandle	hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(!hCDEF)
		return(ret);

	partCode = (**theControl).contrlHilite;
	rScale = (**hCDEF).scaleRect;
	
//----------------------------------------------------------------------------------
// get the thumb rect, calculated from control value
//----------------------------------------------------------------------------------
	getThumbRect(theControl, &rThumb, &rScale);
	
	rScale.left+=17;									// inset for active area
	InsetRect(&rScale,8,5);
	
	p.h = LoWord(param);
	p.v = HiWord(param);
	
	if(PtInRect(p, &rThumb)) {							// click in thumb?
		ret = 1;
		dragThumb(theControl, varCode, param);
	}
	else
	if(PtInRect(p, &rScale)) {							// click in scale?
		if(p.v < rThumb.bottom)
			ret = 2;									// decrease
		else
		if(p.v > rThumb.top)
			ret = 3;									// increase
			
		if((**hCDEF).firstClick)						// seem to get called
			(**hCDEF).firstClick--;						// too often
			
		if((**hCDEF).firstClick || partCode == ret) {
			scaleClick(theControl, varCode, (short)ret);
			Delay(6L, & secs);
		}
	}
	return(ret);
}

//==================================================================================
// Respond to a mouse down in our "thumb", track the mouse to reposition the thumb,
// set a new control value and call the actionProc (if any).
//==================================================================================
static void dragThumb (ControlHandle theControl, short varCode, long param)
{
	short	units,newVal,snap,maxDrag,minDrag;
	Rect	rThumb, rScale;
	Point	p,oldPt;
	
	rScale = (**(CDEFHandle)(**theControl).contrlData).scaleRect;
	getThumbRect(theControl, &rThumb, &rScale);
	
	units = ((rScale.bottom-rScale.top)-21)/12;
	p.h = LoWord(param);
	p.v = oldPt.v = HiWord(param);
	
	if(PtInRect(p, &rThumb)) {
		maxDrag = rScale.bottom - 4 - (rThumb.bottom - p.v);
		minDrag = rScale.top + 4 + (p.v - rThumb.top);
		
		while (StillDown()) {
			if(oldPt.v != p.v) {							// mouse moved
				OffsetRect(&rThumb, 0, (p.v-oldPt.v));		// move the thumb
				if(rThumb.top < rScale.top+5)				// pin to top of scale
					OffsetRect(&rThumb, 0, rScale.top - rThumb.top+5); 
				else
				if(rThumb.bottom > rScale.bottom-5)			// pin to bottom of scale
					OffsetRect(&rThumb, 0, rScale.bottom - rThumb.bottom-5); 
					
				newVal = (rScale.top - rThumb.top)+units*12+5;
				if(newVal != (**theControl).contrlValue) {
					(**theControl).contrlValue = newVal;
					doDraw(theControl, varCode);
					if((**theControl).contrlAction != nil)
						CallPascalL(theControl, 1,(**theControl).contrlAction);
				}
				if(p.v > minDrag && p.v < maxDrag)			// track move
					oldPt.v = p.v;	
			}
			GetMouse(&p);
		}
	}

//----------------------------------------------------------------------------------
// adjust the control value & redraw if necessary
//----------------------------------------------------------------------------------

	if(!(varCode & thumbNoSnap)) {
		snap = (rThumb.top-rScale.top-5)%12;
		if(snap) {
			if(snap >= 7)
				OffsetRect(&rThumb, 0, 12-snap);
			else
				OffsetRect(&rThumb, 0, -snap);
		}
	}
	(**theControl).contrlValue = (rScale.top - rThumb.top)+units*12+5;
	doDraw(theControl, varCode);
	if((**theControl).contrlAction != nil)
		CallPascalL(theControl, 1,(**theControl).contrlAction);
}

//==================================================================================
// Adjust the control after detecting a click in the scale instead of the thumb -
// set a new control value and call the actionProc (if any).
//==================================================================================
static void scaleClick (ControlHandle theControl, short varCode, 
							short partCode)
{
	Rect	rThumb, rScale;
	short	incr,newVal,units;

	rScale = (**(CDEFHandle)(**theControl).contrlData).scaleRect;
	getThumbRect(theControl, &rThumb, &rScale);
	
	units = ((rScale.bottom-rScale.top)-21)/12;
	if(varCode & thumbNoSnap)
		incr = 1;
	else
		incr = 12;
		
	if(partCode == 2) {
		OffsetRect(&rThumb, 0, -incr);
	}
	else 
	if(partCode == 3) {
 		OffsetRect(&rThumb, 0, incr);
	}	

	if(rThumb.top < rScale.top+5)							// pin to top of scale
		OffsetRect(&rThumb, 0, rScale.top - rThumb.top+5); 
	else
	if(rThumb.bottom > rScale.bottom-5)						// pin to bottom of scale
		OffsetRect(&rThumb, 0, rScale.bottom - rThumb.bottom-5);
					
	newVal = (rScale.top - rThumb.top)+units*12+5;
	
	if(newVal != (**theControl).contrlValue) {
		(**theControl).contrlValue = newVal;
		doDraw(theControl, varCode);
		if((**theControl).contrlAction != nil)
			CallPascalL(theControl, partCode,(**theControl).contrlAction);
	}	
}

//==================================================================================
//	Calculate the thumb location from the control value
//==================================================================================

static void getThumbRect(ControlHandle theControl, Rect *rThumb, Rect *rScale)
{
	short	v;
	
	SetRect(rThumb, 0, 0, 23, 11);
	v = (rScale)->bottom - 16 - (**theControl).contrlValue;
	OffsetRect(rThumb, (rScale)->left+18,v);
}

//==================================================================================
//	Draw both the scale and thumb into one bitmap
//==================================================================================

Boolean drawParts (ControlHandle theControl, short varCode, Rect *rThumb)
{

	Rect			scaleRect,partRect,r;
	BitMap			bits, *offBits,*scaleBits;
	PixMapHandle	pmHdl, pmHdl2;
	Boolean			haveGW = false, inColor = false, haveGray = false;
	RGBColor		saveFore, saveBack, grayColor,tingeColor;
	RGBColor		scaleColor = {56797, 56797, 56797};
	Pattern			pat = {0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA};
	CDEFHandle		hCDEF;
	
	char vThumb[] = {	// BitMap data for a 23 x 11 image of thumb
		0x3F, 0xFF, 0xF8, 0x00, 0x40, 0x00, 0x04, 0x00,
		0x7F, 0xFF, 0xFC, 0x00, 0x80, 0x00, 0x02, 0x00,
		0x80, 0x00, 0x02, 0x00, 0x80, 0x00, 0x02, 0x00,
		0x80, 0x00, 0x02, 0x00, 0x80, 0x00, 0x02, 0x00,
		0x7F, 0xFF, 0xFC, 0x00, 0x40, 0x00, 0x04, 0x00,
		0x3F, 0xFF, 0xF8, 0x00 };
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		if((**hCDEF).scalePort == 0)
			return;
		
//----------------------------------------------------------------------------------
// set up our offscreen ports - the one we draw into and the the one that
// contains the scale we drew previously.
//----------------------------------------------------------------------------------

		SetRect(&scaleRect, 0, 0, 42, (**hCDEF).scaleSize*12+21);
			
		if((**hCDEF).qdVers >= gestalt32BitQD12) {
			haveGW = true;
			pmHdl  = getLockedPixels(&(**hCDEF).offPort, (**hCDEF).qdVers);
			pmHdl2 = getLockedPixels(&(**hCDEF).scalePort, (**hCDEF).qdVers);
			if(!pmHdl || !pmHdl2)
				return false;
		}
		
		if(haveGW) {
			offBits   = (BitMap *)*pmHdl;
			scaleBits = (BitMap *)*pmHdl2;
		}
		else {
			offBits   = (BitMap *)&((GrafPtr)(**hCDEF).offPort)->portBits;
			scaleBits = (BitMap *)&((GrafPtr)(**hCDEF).scalePort)->portBits;
		}
		
//----------------------------------------------------------------------------------
// set up our colors
//----------------------------------------------------------------------------------

		if((**hCDEF).pDepth > 2) {
			inColor = true;
			saveColors(&saveFore, &saveBack);
			setPartColor(theControl, cFrameColor, true);
		}

		if((**theControl).contrlHilite == 0xFF) {			// inactive control
			if(inColor) {									// try to draw it in
				if(haveGray = getGray(&grayColor))			// gray
					RGBForeColor(&grayColor);
			}
		}
//----------------------------------------------------------------------------------
// copy the scale into this offport and do 3D stuff if needed
//----------------------------------------------------------------------------------
		
		CopyBits(scaleBits,	
				offBits,
				&scaleRect,
				&scaleRect,
				srcCopy, nil);	

		if(varCode & ctl3D && inColor) {					// draw with 3D effect
			GetForeColor(&saveFore);
			RGBForeColor(&scaleColor);
			if(haveGray) {									// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}

//----------------------------------------------------------------------------------
// fill center of scale with light gray
//----------------------------------------------------------------------------------

			scaleRect.left+=17;
			InsetRect(&scaleRect,9,4);
			SetRect(&r, scaleRect.left+2,scaleRect.top-1,
						scaleRect.right-2,scaleRect.top+1);
			PaintRect(&r);
			SetRect(&r, scaleRect.left+2,scaleRect.bottom-1,
						scaleRect.right-2,scaleRect.bottom+1);
			PaintRect(&r);
			PaintRect(&scaleRect);
			
//----------------------------------------------------------------------------------
// draw shadow along bottom & right
//----------------------------------------------------------------------------------

	  		setPartColor(theControl, cTingeLight, true);
	  		GetForeColor(&tingeColor);
	  		tingeColor.red-=13107;
	  		tingeColor.green-=13107;
	  		tingeColor.blue-=13107;
	  		RGBForeColor(&tingeColor);
			if(haveGray) {									// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
	  		MoveTo(scaleRect.left+2, scaleRect.bottom);
	  		Line(2,0);
	  		Line(0,-1);
	  		Line(2,0);
	  		LineTo(scaleRect.right-1, scaleRect.top);
	  		RGBForeColor(&saveFore);
		}

//----------------------------------------------------------------------------------			
// draw the Thumb into the offPort
//----------------------------------------------------------------------------------
		
	  	EraseRect(rThumb);
		SetRect(&partRect, 0, 0, 23, 11);					// construct a bitmap	
		bits.bounds = partRect;
		bits.rowBytes = 4;									// on the fly
		bits.baseAddr = vThumb;
		
		CopyBits(&bits,										// from bitmap
				offBits,									// to onscreen port
				&partRect,									// from rect
				rThumb,			 							// to rect
				srcCopy, nil);
		
	  	if(varCode & ctl3D  && inColor) {					// draw with 3D effect

			RGBForeColor(&scaleColor);
			if(haveGray) {									// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
	  		InsetRect(rThumb, 1, 3);
	  		PaintRect(rThumb);
	  		MoveTo((*rThumb).left+1, (*rThumb).top-2);
	  		LineTo((*rThumb).right-2, (*rThumb).top-2);
	  		MoveTo((*rThumb).left+1, (*rThumb).bottom+1);
	  		LineTo((*rThumb).right-2, (*rThumb).bottom+1);
	  		
//----------------------------------------------------------------------------------
// draw the 3D shadow in a grayed cTingeColor
//----------------------------------------------------------------------------------

	  		RGBForeColor(&tingeColor);
			if(haveGray) {									// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
	  		MoveTo((*rThumb).left, (*rThumb).bottom-1);
	  		LineTo((*rThumb).right-1, (*rThumb).bottom-1);
	  		LineTo((*rThumb).right-1, (*rThumb).top);
	  		MoveTo((*rThumb).right-2, (*rThumb).bottom-2);
	  		LineTo((*rThumb).right-2, (*rThumb).bottom-2);
	  	}
		if((**hCDEF).qdVers >= gestalt32BitQD13) {
			UnlockPixels(pmHdl);
			UnlockPixels(pmHdl2);
		}
  	}
  	return(haveGray);
}

//==================================================================================
//	Draw the control, it is made up from the scale bitmap and the thumb bitmap
//	which are assembled into an offscreen bitmap in "drawParts".  Now, we blit it
//  to our onscreen port.
//==================================================================================

static void doDraw (ControlHandle theControl, short varCode)
{
	short			scaleSize;
	Rect			offRect,rThumb,rScale;
	RgnHandle		clipRgn;
	BitMap			*offBits;
	PixMapHandle	pmHdl;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	PenState		savePen;
	Boolean			inactive = false, haveGray = false, inColor = false;
	RGBColor		saveFore,saveBack;
	RGBColor		rgbBlack = {0,0,0};
	RGBColor		rgbWhite = {-1,-1,-1};
	CDEFHandle		hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		scaleSize = (**hCDEF).scaleSize;
		rScale = (**hCDEF).scaleRect;
		if((**hCDEF).scalePort == 0 ) {
			makeScale(theControl, varCode, scaleSize);
			if((**hCDEF).scalePort == 0)						// oh, oh
				return;					
		}

//----------------------------------------------------------------------------------	
// first, create or update our GWorld, and set it as our drawing port	
//----------------------------------------------------------------------------------
	
		(**hCDEF).pDepth = getOff(&(**hCDEF).offPort, &(**theControl).contrlRect);
		
		if((**hCDEF).pDepth == 0)								// oh oh
			return;										
		
		if((**hCDEF).qdVers >= gestalt32BitQD12) {				// have a GWorld
			pmHdl  = getLockedPixels(&(**hCDEF).offPort, (**hCDEF).qdVers);
			if(!pmHdl)
				return;											// nuts
		}
		
		clipRgn = NewRgn();	
		GetClip(clipRgn);
		ClipRect(&(**theControl).contrlRect);

//----------------------------------------------------------------------------------					
// Position the thumb rect
//----------------------------------------------------------------------------------
		
		SetRect(&offRect, 0, 0, 42, scaleSize*12+21);
		SetRect(&rThumb, 0, 0, 23, 11);
		OffsetRect(&rThumb, 18, offRect.bottom - 16 - (**theControl).contrlValue);
		
//----------------------------------------------------------------------------------
// Initialize and set for drawing into offScreen port
//----------------------------------------------------------------------------------

	  	if((**theControl).contrlHilite == 0xFF)					// inactive control
			inactive = true;
			
		if((**hCDEF).pDepth > 2) {								// save onscreen	
			inColor = true;										// colors
			saveColors(&saveFore, &saveBack);
		}
		if((**hCDEF).qdVers >= gestalt32BitQD12) {				// we have a pixMap	
			GetGWorld(&savePort, &saveGDev);
			SetGWorld((**hCDEF).offPort, nil);		
			offBits = (BitMap *)*(**hCDEF).offPort->portPixMap;
		}
		else {													// we have a bitMap	
			GetPort((GrafPtr*)&savePort);
			SetPort((GrafPtr)(**hCDEF).offPort);		
			offBits = (BitMap *)&((GrafPtr)(**hCDEF).offPort)->portBits;
		}
		if(inColor) {
			RGBForeColor(&saveFore);
			RGBBackColor(&saveBack);
		}

//----------------------------------------------------------------------------------	
// draw the control parts into the offscreen port
//----------------------------------------------------------------------------------
	
		haveGray = drawParts (theControl, varCode, &rThumb);
	
//----------------------------------------------------------------------------------	
// restore onscreen port & set colors to b&w
//----------------------------------------------------------------------------------

		if((**hCDEF).qdVers >= gestalt32BitQD12) {
			SetGWorld(savePort, saveGDev);
		} else {
			SetPort((GrafPtr)savePort);
		}
		
		if(inColor) {
			RGBForeColor(&rgbBlack);
			RGBBackColor(&rgbWhite);
		}

//----------------------------------------------------------------------------------		
// now copy it to the onscreen port
//----------------------------------------------------------------------------------
	
		CopyBits(offBits,
				&((GrafPtr)savePort)->portBits,
				&offRect,
				&rScale,
				srcCopy, nil);
		
		if((**hCDEF).qdVers >= gestalt32BitQD13)
			UnlockPixels(pmHdl);
				
		if(inColor) {
			restoreColors(&saveFore, &saveBack);
		}

//----------------------------------------------------------------------------------
// if we failed to get an inactive gray color, do it the old way
//----------------------------------------------------------------------------------

		if(inactive & !haveGray) {							// use the old way of drawing
			GetPenState(&savePen);							// an inactive control
			PenMode(patBic);
			PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
			PaintRect(&(**theControl).contrlRect);
			SetPenState(&savePen);
		}
		SetClip(clipRgn);
		DisposeRgn(clipRgn);
	}
}

//==================================================================================
//	Create the scale as a bitmap drawn in an offscreen port. It is built up from
//	bitmap fragments (data in char arrays) and variation codes applied
//==================================================================================

static void makeScale	(ControlHandle theControl, short varCode, short scaleSize)
{
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	Rect			r,scaleRect,bitRect;
	BitMap			bits, *scaleBits;
	PixMapHandle	pmHdl;
	short			h,v,inx,s,e;
	char			c,c1;
	
	char vTop[] = {  	// 25 x 11
	0x00, 0x3E, 0x00, 0x00, 0x00, 0xC1, 0x80, 0x00,
	0x01, 0x1C, 0x40, 0x00, 0x02, 0x6B, 0x20, 0x00,
	0x02, 0xD5, 0xA0, 0x00, 0x02, 0xAA, 0xA0, 0x00, 
	0x02, 0xD5, 0xA0, 0x00, 0x02, 0xAA, 0xA0, 0x00, 
	0x02, 0xD5, 0xA0, 0x00, 0x02, 0xAA, 0xA0, 0x00, 
	0xF2, 0xD5, 0xA0, 0x00};
	
	char vCenter[] = {  // 25 x 12
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0xF2, 0xD5, 0xA0, 0x00};
	
	char vBottom[] = {	// 25 x 10
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0xAA, 0xA0, 0x00, 0x02, 0xD5, 0xA0, 0x00,
	0x02, 0x6B, 0x20, 0x00, 0x01, 0x1C, 0x40, 0x00,
	0x00, 0xC1, 0x80, 0x00, 0x00, 0x3E, 0x00, 0x00};
	
	CDEFHandle	hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		SetRect(&bitRect, 0, 0, 25, 11);					// size of top part
		SetRect(&scaleRect, 0, 0, 42, scaleSize*12+21);		// size of offScreen port
	 	r = scaleRect;
	 	
		inx = getOff(&(**hCDEF).scalePort, &scaleRect);		// get the offScreen port
		
		if(inx && (**hCDEF).scalePort) {					// got it, now set to draw
			if((**hCDEF).qdVers >= gestalt32BitQD12) {		// in it.
				pmHdl = getLockedPixels(&(**hCDEF).scalePort, (**hCDEF).qdVers);
				if(!pmHdl)
					return;									// nuts
			
				GetGWorld(&savePort, &saveGDev);
				SetGWorld((**hCDEF).scalePort, nil);
				scaleBits = (BitMap *)*pmHdl;
			} 
			else {
				GetPort((GrafPtr *)&savePort);
				SetPort((GrafPtr)(**hCDEF).scalePort);
				scaleBits = (BitMap *)&((GrafPtr)(**hCDEF).scalePort)->portBits;
			}
		 	EraseRect(&scaleRect);
			scaleRect.left+=17;							// 1st, blit the top part
			scaleRect.bottom = scaleRect.top+11;
	  		
			bits.rowBytes = 4;
			bits.bounds = bitRect;
			bits.baseAddr = vTop;
			CopyBits(&bits,								// from bitmap
					scaleBits,							// to offscreen port
					&bitRect,							// from rect
					&scaleRect,			 				// to rect
					srcCopy, nil);
			
			bitRect.bottom++;							// center section 1 pixel taller
			for(inx=0;inx<scaleSize;inx++) {			// 2nd, blit the center part(s)
				scaleRect.top=scaleRect.bottom;			// from top to bottom in the
				scaleRect.bottom=scaleRect.top+12;		// offscreen port
				
				bits.bounds = bitRect;
				bits.baseAddr = vCenter;
				CopyBits(&bits,							// from bitmap
						scaleBits,						// to offscreen port
						&bitRect,						// from rect
						&scaleRect,			 			// to rect
						srcCopy, nil);
			}
			scaleRect.top=scaleRect.bottom;
			scaleRect.bottom=scaleRect.top+10;
			bitRect.bottom-=2;							// center section 2 pixels shorter
			
			bits.bounds = bitRect;						// 3rd, blit the bottom part
			bits.baseAddr = vBottom;
			CopyBits(&bits,								// from bitmap
					scaleBits,							// to offscreen port
					&bitRect,							// from rect
					&scaleRect,			 				// to rect
					srcCopy, nil);
			
			if(!(varCode & scaleBlank)) {
				TextFont(geneva);						// draw numbers next to
				TextSize(9);							// tick marks,
				v = scaleRect.bottom-6;					// which start here
				h = scaleRect.left-10;
				c = c1 = '0';
				for(inx=0;inx<scaleSize+1;inx++) {
					if(!(inx%10) && inx) {
						c = '0';
						c1++;
					}
					MoveTo (h, v);
					DrawChar(c);
					if(inx>9) {
						h-=6;
						MoveTo(h,v);
						DrawChar(c1);
						h+=6;
					}
					v-=12;
					c++;
				}
			}
			if(varCode & scaleBlank) {					// erase tick marks
				r.right-=19;
				EraseRect(&r);
			}
			if(varCode & scaleNoFill) {					// erase the center
				scaleRect.top-=(scaleSize*12+11);
				InsetRect(&scaleRect,9,4);
				SetRect(&r, scaleRect.left+2,scaleRect.top-1,
							scaleRect.right-2,scaleRect.top+1);
				EraseRect(&r);
				SetRect(&r, scaleRect.left+2,scaleRect.bottom-1,
							scaleRect.right-2,scaleRect.bottom+1);
				EraseRect(&r);
				EraseRect(&scaleRect);
			}
			if((**hCDEF).qdVers >= gestalt32BitQD12) {
				SetGWorld(savePort, saveGDev);
			} else {
				SetPort((GrafPtr)savePort);
			}
			if((**hCDEF).qdVers >= gestalt32BitQD13)
				UnlockPixels(pmHdl);
		}
	}
	return;
}
