/**************************************************************************

	CWFailure.cp
	
	Mark E. Anderson
	8/8/94

CW currently can't output to stderr.  What follows is a kludge that puts the message 
in a window of the Scriptable Text Editor through AppleScript.
You *must* open Scriptable Text Editor for this code to work.
	
**************************************************************************/

#if __MWERKS__  && qDebug

#ifndef __CWFAILURE__
#include <CWFailure.h>
#endif




#pragma segment GDebug

XAppleEvent::XAppleEvent( OSType appid, OSType eventclass, OSType eventid )
{
	AEAddressDesc 	address = {typeNull, nil}; 
	OSErr			err;

	fEvent.descriptorType = typeNull; 
	fEvent.dataHandle = nil; 
	
	err = AECreateDesc (typeApplSignature, (Ptr) &appid, sizeof (appid), &address);
	if ( err == noErr )
		err = AECreateAppleEvent ( eventclass, eventid, &address, 0, 0, &fEvent);
	if ( err != noErr )
		ProgramBreak("Failed in XAppleEvent constructor");	

	AEDisposeDesc (&address);		
	
}


XAppleEvent::XAppleEvent( OSType appid, OSType eventclass, OSType eventid, XAEApplication* directObject )
{
	AEAddressDesc 	address = {typeNull, nil}; 
	OSErr			err;

	fEvent.descriptorType = typeNull; 
	fEvent.dataHandle = nil; 
	
	err = AECreateDesc (typeApplSignature, (Ptr) &appid, sizeof (appid), &address);
	if ( err == noErr )
		err = AECreateAppleEvent ( eventclass, eventid, &address, 0, 0, &fEvent);
	if ( err == noErr )
		err = AEPutParamDesc (&fEvent, keyDirectObject, directObject->GetObject() );
	if ( err != noErr )
		ProgramBreak("Failed in XAppleEvent constructor");	

	AEDisposeDesc (&address);		
	
}

OSErr	XAppleEvent::AddIndirectObject( AEKeyword theKeyword, DescType type, Ptr thing, Size size )
{
	OSErr			err;
	AEDesc			indirect = {typeNull, nil};
	
	
	err = AECreateDesc(type, thing, size, &indirect);

	if ( err == noErr )
		err = AEPutParamDesc (&fEvent, theKeyword, &indirect);	
					
	if ( err != noErr )
		ProgramBreak("Failed in AddIndirectObject");

	AEDisposeDesc (&indirect);	
	
	return err;	
}

OSErr	XAppleEvent::Send( void )
{
	AppleEvent 		reply = {typeNull, nil};
	OSErr			err;
	
		err = AESend ( &fEvent, &reply,  kAENoReply , kAENormalPriority,  kNoTimeOut, nil, nil);	

	AEDisposeDesc (&reply);		
	return err;
}


XAppleEvent::~XAppleEvent( void )
{
	AEDisposeDesc (&fEvent);			
}

OSErr	XAppleEvent::GetResponseFromReply( OSType desiredType, Ptr thing, Size size )
{

	AppleEvent 		reply = {typeNull, nil};
	OSErr 			err;
	DescType 		actualtype;
	Size 			actualsize;


	err = AESend ( &fEvent, &reply,  kAEWaitReply + kAENeverInteract, 
										kAENormalPriority,  120, nil, nil);	
	if ( err == noErr )
		err = AEGetParamPtr ( &reply, keyDirectObject, desiredType, 		
						&actualtype, thing, size, &actualsize);
	if ( err != noErr )
		ProgramBreak("Failed in GetResponseFromReply");
		

	AEDisposeDesc (&reply);	
	
	return err;		
}

long	XAppleEvent::GetNumOfThings( void )
{

	OSErr 			err;
	long			things = badReturnValue;

	err = GetResponseFromReply( typeLongInteger, (Ptr) &things, sizeof (things) );

	if ( err != noErr )
		ProgramBreak("Failed in GetNumOfThings");
	
	return things;		
}

Boolean	XAppleEvent::DoesThingExist( void )
{

	OSErr 			err;
	Boolean			thingExists = false;

	err = GetResponseFromReply( typeBoolean, (Ptr) &thingExists, sizeof (thingExists) );

	if ( err != noErr )
		ProgramBreak("Failed in DoesThingExist");
	
	return thingExists;		
}


XAEApplication::XAEApplication ( void )
{

	fAppObj.descriptorType = typeNull; 
	fAppObj.dataHandle = nil; 

}


XAEApplication::~XAEApplication(void) 
 { 

	AEDisposeDesc (&fAppObj);

 }


XAEWindow::XAEWindow ( void ) : XAEApplication( )
{
	OSErr    	myErr;
	AEDesc   	windNumDesc = {typeNull, nil}; 
	long 		windowNumber;
	
	fWindObj.descriptorType = typeNull; 
	fWindObj.dataHandle = nil; 
	//we only deal with the frontmost window 
	windowNumber = 1;
	//the container is Null
	myErr = AECreateDesc(typeLongInteger,(Ptr)&windowNumber, sizeof(windowNumber), &windNumDesc);
	myErr = CreateObjSpecifier ( cWindow, &fAppObj, formAbsolutePosition, &windNumDesc, true, &fWindObj );

}

XAEWindow::~XAEWindow(void) 
 { 

	AEDisposeDesc (&fWindObj);

 }



XAEParagraph::XAEParagraph ( long paraNumber ) : XAEWindow( )
{

	OSErr    	myErr;
	AEDesc   	paraNumDesc = {typeNull, nil}; 

	fParaObj.descriptorType = typeNull; 
	fParaObj.dataHandle = nil; 
	//the container is Null
	myErr = AECreateDesc(typeLongInteger,(Ptr)&paraNumber, sizeof(paraNumber), &paraNumDesc);
	myErr = CreateObjSpecifier ( cParagraph, &fWindObj, formAbsolutePosition, &paraNumDesc, true, &fParaObj );

}

XAEParagraph::~XAEParagraph(void) 
 { 
	AEDisposeDesc (&fParaObj);
	 
 }


Boolean sendMessageToWrite (OSType appid, OSType eventclass, OSType eventid, char * txt) 
{
	
	long			paragraphNum;
	Boolean			result = false;
	OSErr 			err;
	OSType			indirect = cParagraph;
	
	XAEWindow		windObj;
	XAppleEvent		count( appid,  kAECoreSuite, kAECountElements, &windObj );
	count.AddIndirectObject( keyAEObjectClass, typeType, (Ptr)&indirect, sizeof(indirect) );
	paragraphNum = count.GetNumOfThings();

	if ( paragraphNum != badReturnValue )
	{
				//make an apple event for changing the text in the last paragraph
		XAEParagraph	paraObj( paragraphNum );
		XAppleEvent		writeIt( appid, eventclass, eventid, &paraObj );
		writeIt.AddIndirectObject( keyAEData, typeChar, txt, strlen(txt) );
		err = writeIt.Send();
		if (noErr == err )
		{
			//make a apple event for selecting the text in the last paragraph (so it scrolls)
			paragraphNum = count.GetNumOfThings();// call because the number of paras has changed
			XAEParagraph	paraObj2( paragraphNum - 1 );	//we want to select the last line
															//with a word; not the return character
			XAppleEvent	selectIt( appid,  kAEMiscStandards,  kAESelect, &paraObj2 );
			err = selectIt.Send();
		}
	}
	

	return noErr == err;

} /*sendMessageToWrite*/

Boolean FindAProcess( OSType signature )
{
	Boolean				result = false;
	ProcessSerialNumber	PSN;
	ProcessInfoRec		infoRec;
	PSN.highLongOfPSN = 0;
	PSN.lowLongOfPSN = kNoProcess;
	OSType 				application = 'APPL';
	infoRec.processInfoLength = sizeof(ProcessInfoRec);
	infoRec.processName = nil;
	infoRec.processAppSpec = nil;

	while (GetNextProcess( &PSN ) == noErr )
	{
		if ( GetProcessInformation( &PSN, &infoRec ) == noErr )
			if ( (infoRec.processType == application) && 
				(infoRec.processSignature == signature ) )
			{
				result = true;
				break;
			}
	}	
	return result;
}

Boolean AppleEventsAvailable( void )
{
	static Boolean firstTime = true;
	OSErr err;
	
	if ( firstTime  ) 	//do the following just once; if we come back here it means we 
						//don't have System 7; I don't know if that's even possible with MA 3.1
	{
		firstTime = false;	
		if ( TrapAvailable(_Gestalt) )
		{
			long	sysVersion;
			err = Gestalt( gestaltSystemVersion, &sysVersion );
			if ( err )
			{
					ProgramBreak( "\pGestalt failed." );
					return false;
			}
			if ( LoWord(sysVersion) < 0x0700 )
			{
					ProgramBreak( "\pUsing qDebug requires System 7." );
					return false;
			}	
		}
		else
		{
				ProgramBreak( "\pUsing qDebug requires System 7." );
				return false;
		}
	}
	else 
		return false; //we already know something's wrong
	
	return true;  //everything's OK
}

int CWfprintf(FILE * /*ignore*/, const char * format, ...)
{

	va_list	ap = NULL;
	int		result;
	
	//the following 2 strings *should* be long enough since only a line at a time is being printed
	//if you want to use this code as a basis for something else, put error checking below, just
	//before the concatenation
    char 	tmp[512];	//temp storage for output string
    static char	str[512];	//we will concatenate strings until we find a return character
    
    char	fail[] = "\pvsprintf failed\n";
    long len = 0;	//length of string we are concatenating 
	
	Boolean			haveApplescript = true; //set to false if you want to output to Jasik's debugger
	static Boolean	hasAppleEvents = false; 
	static Boolean	firstTime = true;
	OSErr	err;
	OSType	signature = 'quil'; 	//Although this code doesn't use unusual AEs (mostly from 
									//core suite), the only app I know that will work is Scriptable 
									//Text Editor (which you will find on the CD with the AppleScript
									//installation stuff).

	if ( !hasAppleEvents  )
		if ( (hasAppleEvents = AppleEventsAvailable()) == false )
			return 0;
			
	if ( !FindAProcess( signature )  )
	{
		if (firstTime)
		{
			ProgramBreak( "\pLaunch Scriptable Text Editor to display events, etc." );
		 	firstTime = false;
		 }
		return 0;
	}
	//well, it's running, but does it have a window?   
	XAEWindow		windObj; //looking for a window
	XAppleEvent 	exists( signature,  kAECoreSuite, kAEDoObjectsExist, &windObj );

	if ( !exists.DoesThingExist() )
	{
	//The app's running but there isn't a window.  Let's make it easy and open one ourselves.
		OSType 			indirect = cWindow;
		//This AE doesn't have a direct parameter
		XAppleEvent 	make( signature,  kAECoreSuite, kAECreateElement );
		make.AddIndirectObject( keyAEObjectClass, typeType, (Ptr)&indirect, sizeof(indirect) );
		err = make.Send();
	}
	
	//convert to a string whatever was passed to fprintf
    va_start(ap, format );    
	result = vsprintf(tmp, format, ap );
	va_end(ap);
	if ( result >= 0 ) //we were able to convert the fprintf "string" into a C string
	{
		strcat( str, tmp ); 
		len = strlen(str); 

		if ( 	str[len - 1] == 0x0A || 	// vprintf by default converts the \n to a 0x0A
				str[len - 1] == 0x0D	)   // unless you have #pragma mpwc_newline on and 
											// then it's a 0x0D; we'll cover both possibilities
			str[len - 1] = 0x0D;	//we need to change it back to a \n for STE
		else
			return (result);  	//don't send the AE yet; we haven't received the \n
									
	}
	else
		strcpy(str, fail);  //something went wrong

	
	if (haveApplescript)
		sendMessageToWrite ( signature , kAECoreSuite, kAESetData, str );
	else
   		ProgramBreak(str); //send to Jasik's debugger

	str[0] = NULL;			//initialize str
	
	return (result);
	
}
#endif //__MWERKS__  && qDebug
