/******************************************************************************

	CStandApp.h
 
 	C++ declarations for general standalone application
 
	Copyright  1992 NeoLogic Systems	All rights reserved.
	
******************************************************************************/

#ifndef __CSTANDAPP__
#define __CSTANDAPP__ 1

#include <NeoTypes.h>
#include CNeoAppNativeH

class CNeoIndexIterator;

#define kNeoStandFileType	'Ne5d'
#define kNeoStandSig		'Neo5'

#ifdef qNeoThreads
#include CNeoThreadNativeH

typedef enum {
	kDie,
	kAlive
} health;

#define kMaxThreads		5

typedef struct {
	short				phase;		/* Phase of work being done				*/
	health				state;		/* Healthiness of this thread			*/
	CNeoWindow *		window;		/* Window this thread is working for	*/
	CNeoThreadNative *	thread;		/* Pointer to thread					*/
} ThreadInfo;

class CBenchThread : public CNeoThreadNative
{
public:
						CBenchThread(ThreadInfo *aInfo, const NeoThreadType aType = kNeoCoopThread, void **aArg = nil, const Size aStackSize = kNeoDefaultStackSize, const NeoThreadOptions aOptions = (kNeoThreadNewSuspend + kNeoThreadUsePremade + kNeoThreadCreate));
	virtual long		run(void);

	virtual void		handleSwapIn(void);
	virtual void		handleSwapOut(void);

protected:
	Boolean				fSetTimer;
	TMTask				fTimer;
	ThreadInfo *		fInfo;
};
#endif

/*
 * A NeoPhase structure, containing a target string,
 * number of iterations done, and a soFar in milliseconds.
 */
 
enum phases {
	kNoPhase	= -1,
	kMinPhase	= 0,
	kInsert		= kMinPhase,
	kRandomly,
	kSerially,
	kChange,
	kDelete,
	kMaxPhase	= kDelete
};

// Indices into the array going across.	There are three of them
enum cols {
	kMinCol		= 0,
	kTargetCol	= kMinCol,
	kSoFar,
	kPerRecord,
	kTotal,
	kMaxCol		= kTotal
};

enum states {
	kStop,
	kStart
};

typedef struct NeoPhase {
	Boolean	dirty;		/* Do values for this phase need updating?	*/
#ifdef qNeoThreads
	short	threadCount;/* Number of threads to use for this phase	*/
#endif
#if __MWERKS__
	Boolean	finished;	/* TRUE if target operations have been done */
#endif
	long	target;		/* Copied from TextBox field in main pane   */
	long	delta;		/* Initial count							*/
	long	done;		/* Number of operations done so far			*/
	long	committed;	/* Number of operations committed so far	*/
	long	soFar;		/* Total time units in Milliseconds so far	*/
} NeoPhase;

class CStandApp : public CNeoAppNative {
public:
						/** Instance Methods **/
						CStandApp(void);
						~CStandApp(void);
	static void			NewHandler(void);

	virtual void		doChores(void);
	void				doSomeWork(const short aPhase, TMTask *aTimer);
	virtual void		exit(void);
	short				getPhase(void) const {return fPhase;}
	long				getPhaseTarget(const short aPhase) const {return fPhaseInfo[aPhase].target;}
	Boolean				getState(void) const {return fState;}
	void				getTime(long aValue, char *aString);
	void				getTargetTotals(void);
#ifdef qNeoThreads
	void				killThreads(void);
#endif
	void				setPhase(const short aPhase);
	void				setState(const Boolean aState);
	void				setDirty(const Boolean aDirty = TRUE) {fDirty = aDirty;}

						/** Instance Variables **/
protected:
	Boolean				fDirty;
	Boolean				fState;						// Running state
	Boolean				fRefresh;					// Refresh the screen?
	short				fPhase;						// The current phase
	short				fIndex;						// Index into the column
	long				fValue[3];					// 3 columns - soFar, PerRecord, Total
	CNeoIndexIterator *	fIterator;					// For traversing index of objects
	NeoPhase			fPhaseInfo[kMaxPhase +1];	// The main phases
#ifdef qNeoThreads
	short				fThreadCount;				// Current number of slave threads
	ThreadInfo			fThreadInfo[kMaxThreads];	// Info used by each thread
#endif
};
#endif
