// ----------------------------------------------------------------------------------
// MenuHandler.c
// ----------------------------------------------------------------------------------
// Handles all menu commands
//
// This demo is copywrite 1994 by LexTek Internation.  Feel free to use it for
// whatever purpose you wish.  The SpellWright Library may not be copied or 
// distributed, except when linked to your application that adds significant features
// to the code.  ie. you can't take the library, make a new library and sell it.  You
// can, however, use it without royalties in applications or other such projects.
//
// LexTek International
// 2255 N. University Parkway, Suit 15
// Provo, UT 84604
// (801) 375.8332  Phone
// (801) 375.7654  Fax 
//
// ----------------------------------------------------------------------------------
// History:
//	8/26/94		CG	Original
//	9/3/94		CG	Added Dictionary Menu

#include "EventLoop.h"
#include "MenuHandler.h"
#include "Globals.h"
#include "SpellHandler.h"
#include "TEHandler.h"
#include "StyleHandler.h"
#include "WindowHandler.h"


void DoAppleMenu(short item);
void ChooseApple(short itemNumber);
void ChooseFile(short item);
void ChooseEdit(short item);
void ChooseUtility(short item);
void MenuHandler(short menuNumber,short itemNumber);
void AdjustMenus(void);

// ----------------------------------------------------------------------------------

extern void PrintWindow( void );
extern void ErrMsg(Str255 msg);
extern void ChooseSlim(short item);


// ----------------------------------------------------------------------------------
// DoAppleMenu
// ----------------------------------------------------------------------------------
// Handles an Apple Menu item.

void DoAppleMenu(short item)
{
	GrafPtr		savePort;
	short		daRefNum;
	Str255		daName;
	
	GetPort(&savePort);
	
	GetItem(GetMHandle(mApple), item, daName);
	daRefNum = OpenDeskAcc(daName);
	
	SetPort(savePort);
	
} // DoAppleMenu


// ----------------------------------------------------------------------------------
// ChooseApple
// ----------------------------------------------------------------------------------
// Handles the Apple menu.  It either chooses the about item or one of the general
// apple menu items.

void ChooseApple(short itemNumber)
{
	short item;
	
	if(itemNumber == iAbout) 
	{
		item = Alert(ABOUT_BOX_ID,nil);
	} else 
	{
		DoAppleMenu(itemNumber);
	} // if/else
} // ChooseApple

// ----------------------------------------------------------------------------------
// ChooseFile
// ----------------------------------------------------------------------------------
// Handles the File menu.

void ChooseFile(short item)
{
	switch(item) {
		case iNew:			CreateDocWindow();
							break;
		case iOpen:			OpenDocWindow();
							break;
		case iClose:		CloseDocWindow(FrontWindow());
							break;
		case iSave:			SaveDocWindow(FrontWindow());
							break;
		case iSaveAs:		SaveDocWindowAs(FrontWindow());
							break;
		case iRevert:
							break;
		case iPageSetup:
							break;
		case iPrint:		// PrintWindow();
							break;
		case iQuit:			gQuit = true;
							break;
		default:			break;
	} // switch
} // ChooseFile

// ----------------------------------------------------------------------------------
// DoCut
// ----------------------------------------------------------------------------------
// Handle Cut menu

void DoCut(TEHandle theTE)
{
	long total, contig;
	
	WinDocRecord	**theData;
	
	theData = (WinDocRecord**) GetWRefCon(gDocWindow);
	
	if (ZeroScrap() == noErr)
	{
		PurgeSpace(&total, &contig);
		TECut(theTE);
		(**theData).dirty = TRUE;
		
		if (TEToScrap() != noErr)
		{
			(void) ZeroScrap();
		}
	}
	ShowSelect(theTE, (**theData).DocVScroll);
	SetScrollBar();

}

// ----------------------------------------------------------------------------------
// DoCopy
// ----------------------------------------------------------------------------------
// Handle Copy menu

void DoCopy(TEHandle theTE)
{
	
	WinDocRecord	**theData;
	
	theData = (WinDocRecord**) GetWRefCon(gDocWindow);
	
	if (ZeroScrap() == noErr)
	{
		TECopy(theTE);
				
		if (TEToScrap() != noErr)
		{
			(void) ZeroScrap();
		}
	}
	
	ShowSelect(theTE, (**theData).DocVScroll);
	SetScrollBar();
}

// ----------------------------------------------------------------------------------
// DoPaste
// ----------------------------------------------------------------------------------
// Handle Paste menu

void DoPaste(TEHandle theTE)
{
	Handle aHandle;
	long oldSize, newSize;
	OSErr saveErr;
	
	WinDocRecord	**theData;
	
	theData = (WinDocRecord**) GetWRefCon(gDocWindow);
	
	aHandle = (Handle) TEGetText(theTE);
	oldSize = GetHandleSize(aHandle);
	newSize = oldSize + TEGetScrapLen();
	
	SetHandleSize(aHandle, newSize);
	saveErr = MemError();
	SetHandleSize(aHandle, oldSize);
	
	if (saveErr != noErr)
		return;
		
	if (TEFromScrap() != noErr)
		return;
		
	TEPaste(theTE);
	(**theData).dirty = TRUE;

	ShowSelect(theTE, (**theData).DocVScroll);
	SetScrollBar();
}


// ----------------------------------------------------------------------------------
// ChooseEdit
// ----------------------------------------------------------------------------------
// Handles the Edit menu.  Right now it tries a system edit and then does a TE edit.

void ChooseEdit(short item)
{	TEHandle	theTE;

	if ( !SystemEdit(item-1) )		// default to the system
	{	
		// only edit with a front window
		if (!gDocWindow)
			return;
			
		theTE = GetTE(gDocWindow);
	
		switch (item)
		{	case iCut:		DoCut( theTE );
							break;
		
			case iCopy:		DoCopy( theTE );
							break;
			
			case iPaste:	DoPaste( theTE );
							break;
			
			case iClear:	TEDelete( theTE );
							break;
	
			default:		break;
		} // switch
		
		SetScrollBar( );
	} // if
} // ChooseEdit

// ----------------------------------------------------------------------------------
// ChooseUtility
// ----------------------------------------------------------------------------------
// Handles the Utility menu.

void ChooseUtility(short item)
{
	switch (item)
	{	case iFind:			break;
		case iSpell:		MakeSpellWindow();
							break;
		case iThesaurus:	break;
		case iOptions:		SpellOptions();
		default:			break;
	} // switch
} // ChooseUtility


// ----------------------------------------------------------------------------------
// MenuHandler
// ----------------------------------------------------------------------------------
// Routes menu commands to the proper menu choice.  ie. this handles the menu and
// then sends the item to that menu's handler.  This makes it easy to add new menu
// items.

void MenuHandler(short menuNumber,short itemNumber)
{
	if(menuNumber != 0)
		switch(menuNumber) {
			case mApple : 	ChooseApple(itemNumber);
							break;
			case mFile :	ChooseFile(itemNumber);
							break;
			case mEdit :	ChooseEdit(itemNumber);
							break;
			case mUtilities:
							ChooseUtility(itemNumber);
							break;
			case mFont:		ChooseFont(itemNumber);
							break;
			case mStyle:	ChooseStyle(itemNumber);
							break;
			default :		break;
		} // switch
} // MenuHandler

// ----------------------------------------------------------------------------------
// AdjustMenus	-- enable/disable menus
// ----------------------------------------------------------------------------------
// Adjust the Menus.  Right now I don't do a whole lot

void AdjustMenus(void)
{
	WindowPtr	window;
	MenuHandle	menu;

	window = FrontWindow();		// we can adjust based upon the front window

	// Update File Menu
	menu = GetMHandle(mFile);
	
	// if there is a window enable close command
	if ( window != nil )
		EnableItem(menu, iClose);
	else
		DisableItem(menu, iClose);

	// Update Edit Menu
	menu = GetMHandle(mEdit);
	if ( ((WindowPeek) window)->windowKind < 0 ) 
	{	// enable commands for system windows
		EnableItem(menu, iUndo);
		EnableItem(menu, iCut);
		EnableItem(menu, iCopy);
		EnableItem(menu, iClear);
		EnableItem(menu, iPaste);
	} else {					
		DisableItem(menu, iUndo);
		DisableItem(menu, iCut);
		DisableItem(menu, iCopy);
		DisableItem(menu, iClear);
		DisableItem(menu, iPaste);
	} // if/else

} // AdjustMenus

